import Loading from "@/components/Loading/Loading";
import { FailedFetchBasicPagesError } from "@/errors";
import DefaultLayout from "@/layouts/DefaultLayout/DefaultLayout";
import { mdxComponents } from "@/lib/mdx-components";
import { fetchBasicPages } from "@/services/content-service";
import { MDXClient } from "next-mdx-remote-client";
import { serialize } from "next-mdx-remote-client/serialize";
import { NextSeo } from "next-seo";
import { useRouter } from "next/router";

export async function getStaticPaths() {
  const res = await fetchBasicPages(["path"]);

  if (!res.ok) {
    throw new FailedFetchBasicPagesError(await res.text());
  }

  const pages = (await res.json()).data;

  return {
    paths: pages.map((page) => ({
      params: {
        path: [page.path ?? ""],
      },
    })),
    fallback: true,
  };
}

export async function getStaticProps({ params }) {
  const { path } = params;

  const res = await fetchBasicPages([], {
    path: {
      _eq: path?.join("/") ?? null,
    },
  });

  if (!res.ok) {
    throw new FailedFetchBasicPageError(path, await res.text());
  }

  const page = (await res.json()).data.at(0);

  if (!page) {
    return {
      notFound: true,
      revalidate: 60,
    };
  }

  const { content, title } = page;
  const mdxSource = await serialize({ source: content });

  return {
    props: {
      title,
      mdxSource,
    },
    revalidate: 60,
  };
}

export default function BasicPage({ title, mdxSource }) {
  const { isFallback } = useRouter();

  if (isFallback) {
    return (
      <DefaultLayout>
        <Loading />
      </DefaultLayout>
    );
  }

  if (!mdxSource || "error" in mdxSource) {
    return <p>Something went wrong: {mdxSource?.error ?? "???"}</p>;
  }

  return (
    <DefaultLayout>
      <NextSeo
        title={title}
        description={undefined}
        openGraph={{
          title,
          description: undefined,
        }}
      />
      <div>
        <MDXClient {...mdxSource} components={mdxComponents} />
      </div>
    </DefaultLayout>
  );
}
